@@ -7,6 +7,7 @@ import pandas as pd
 import pytest
 import pytz
 from dateutil.tz import tzutc
+from pint import DimensionalityError
 from pint_pandas import PintArray
 from pytz import UnknownTimeZoneError
 
@@ -54,6 +55,16 @@ def test_timestamp_series_create_from_lists():
     assert ts.unit is None
 
 
+def test_timestamp_series_create_from_lists_mismatching_length():
+    timestamps = [
+        dt.datetime(2020, 3, 1, 15, 0, 0),
+        dt.datetime(2020, 3, 1, 16, 0, 0),
+    ]
+    values = [-1,]
+    with pytest.raises(ValueError):
+        TimestampSeries.create_from_lists(timestamps, values)
+
+
 def test_timestamp_series_properties_time_range_default(default_timestamp_series):
     assert default_timestamp_series.time_range == (
            pd.Timestamp('2020-01-01T00:00:00').tz_localize('CET').to_pydatetime(),
@@ -184,6 +195,36 @@ def test_timestamp_series_as_pd_series_empty(empty_timestamp_series):
     )
 
 
+def test_timestamp_series_as_pd_series_include_nan():
+    ts = TimestampSeries(
+        pd.Series(PintArray([0., np.nan, 2.], dtype=ureg.parse_units('m')),
+                  index=pd.date_range('2020-01-01', freq='D', periods=3, tz='CET')
+                  ),
+        time_zone='CET', unit='m', freq='D'
+    )
+    pd.testing.assert_series_equal(
+        ts.as_pd_series(include_nan=True),
+        pd.Series([0., np.nan, 2.],
+                  index=pd.date_range('2020-01-01', freq='D', periods=3, tz='CET')
+                  ),
+    )
+
+
+def test_timestamp_series_as_pd_series_exclude_nan():
+    ts = TimestampSeries(
+        pd.Series(PintArray([0., np.nan, 2.], dtype=ureg.parse_units('m')),
+                  index=pd.date_range('2020-01-01', freq='D', periods=3, tz='CET')
+                  ),
+        time_zone='CET', unit='m', freq='D'
+    )
+    pd.testing.assert_series_equal(
+        ts.as_pd_series(include_nan=False),
+        pd.Series([0., 2.],
+                  index=pd.to_datetime(['2020-01-01', '2020-01-03']).tz_localize('CET')
+                  ),
+    )
+
+
 def test_timestamp_series_repr_default(default_timestamp_series):
     from pandas import Series, DatetimeIndex
     from numpy import array
@@ -210,15 +251,75 @@ def test_timestamp_series_str_empty(empty_timestamp_series):
     assert "Time zone: " in str(empty_timestamp_series)
 
 
+def test_map_dimensionless(default_timestamp_series):
+    mapped_ts = default_timestamp_series.map(
+        lambda x: max(x, 1.)
+    )
+    assert mapped_ts.values == [1., 1., 2.]
+    assert mapped_ts._series.dtype.units == ureg.Unit('m')
+
+
+def test_map_with_dimension(default_timestamp_series):
+    mapped_ts = default_timestamp_series.map(
+        lambda x: max(x, 1. * ureg.Unit('m')),
+        dimensionless=False,
+    )
+    assert mapped_ts.values == [1., 1., 2.]
+    assert mapped_ts._series.dtype.units == ureg.Unit('m')
+
+
+def test_aggregate(default_timestamp_series):
+    assert default_timestamp_series.aggregate(np.min) == 0. * ureg.Unit('m')
+
+
+def test_sum(default_timestamp_series):
+    assert default_timestamp_series.sum() == 3. * ureg.Unit('m')
+
+
+def test_mean(default_timestamp_series):
+    assert default_timestamp_series.mean() == 1. * ureg.Unit('m')
+
+
+def test_round_no_unit():
+    timestamps = [
+        pd.Timestamp('2020-01-01T00:00:00').tz_localize(None),
+        pd.Timestamp('2020-01-02T00:00:00').tz_localize(None),
+    ]
+    values = [0.49, -1.51]
+    ts = TimestampSeries.create_from_lists(timestamps, values, time_zone='infer')
+    ts.round(0)
+    assert ts.values == [0., -2.]
+
+
+def test_round_with_unit():
+    timestamps = [
+        pd.Timestamp('2020-01-01T00:00:00').tz_localize(None),
+        pd.Timestamp('2020-01-02T00:00:00').tz_localize(None),
+    ]
+    values = [0.49, -1.51]
+    ts = TimestampSeries.create_from_lists(timestamps, values, time_zone='infer',
+                                           unit='m')
+    ts.round(0)
+    assert ts.values == [0., -2.]
+    assert ts._series.dtype == ureg.Unit('m')
+
+
 def test_timestamp_series_append_default(default_timestamp_series):
     assert default_timestamp_series.append(3.).last == (
         pd.Timestamp('2020-01-04T00:00:00').tz_localize('CET').to_pydatetime(), 3.
     )
 
 
-def test_timestamp_series_append_empty(empty_timestamp_series):
+def test_timestamp_series_append_empty():
+    ts = TimestampSeries(pd.Series([], dtype=float, index=pd.DatetimeIndex([])), freq='10s')
     with pytest.raises(ValueError):
-        empty_timestamp_series.append(1.)
+        ts.append(1.)
+
+
+def test_timestamp_series_append_no_freq():
+    ts = TimestampSeries(pd.Series([1.], dtype=float, index=pd.DatetimeIndex(['2020-01-01'])))
+    with pytest.raises(ValueError):
+        ts.append(1.)
 
 
 def test_timestamp_series_prepend_default(default_timestamp_series):
@@ -341,6 +442,27 @@ def test_create_timestamp_series_mismatching_freq():
         TimestampSeries.create_from_lists(timestamps, values, freq='10Min')
 
 
+def test_create_timestamp_series_mismatching_compatible_unit():
+    ts = (
+        TimestampSeries(series=pd.Series(PintArray([0., 1., 2.], dtype='km'), index=[
+            pd.Timestamp('2020-01-01T00:00:00').to_pydatetime(),
+            pd.Timestamp('2020-01-02T00:00:00').to_pydatetime(),
+            pd.Timestamp('2020-01-03T00:00:00').to_pydatetime(),
+        ]), unit='m')
+    )
+    assert ts.unit == ureg.Unit('m')
+    assert ts.values == [0., 1000., 2000.]
+
+
+def test_create_timestamp_series_mismatching_incompatible_unit():
+    with pytest.raises(DimensionalityError):
+        TimestampSeries(series=pd.Series(PintArray([0., 1., 2.], dtype='m'), index=[
+            pd.Timestamp('2020-01-01T00:00:00').to_pydatetime(),
+            pd.Timestamp('2020-01-02T00:00:00').to_pydatetime(),
+            pd.Timestamp('2020-01-03T00:00:00').to_pydatetime(),
+        ]), unit='km/h')
+
+
 def test_create_timestamp_series_invalid_time_zone():
     illegal_tz = 'Europe/Nantes'
     with pytest.raises(UnknownTimeZoneError):
@@ -365,6 +487,15 @@ def test_create_timestamp_series_invalid_unit():
                         unit=illegal_unit)
 
 
+def test_create_timestamp_series_invalid_index():
+    with pytest.raises(ValueError):
+        TimestampSeries(series=pd.Series([0., 1., 2.], index=[
+            pd.Timestamp('2020-01-03T00:00:00').to_pydatetime(),
+            pd.Timestamp('2020-01-02T00:00:00').to_pydatetime(),
+            pd.Timestamp('2020-01-01T00:00:00').to_pydatetime(),
+        ]))
+
+
 def test_create_timestamp_series_valid_unit_str():
     ts = TimestampSeries(series=pd.Series([], index=pd.DatetimeIndex([])),
                          unit='second')
@@ -435,6 +566,8 @@ def test_timestamp_series_add_timestamp_series_different_tz(default_timestamp_se
     assert default_timestamp_series._series.index.union(
         add_ts._series.index).tolist() == result.timestamps
     assert result.values == [0., 0., 1., 1., 2., 2.]
+    assert result.time_zone == pytz.timezone('CET')
+    assert result._series.index.tzinfo == result.time_zone
 
 
 def test_timestamp_series_add_timestamp_series_different_unit(default_timestamp_series):
@@ -485,6 +618,33 @@ def test_timestamp_series_add_pandas_series():
     )
 
 
+def test_timestamp_series_add_pandas_series_different_time_zones():
+    ts = TimestampSeries(
+        pd.Series(np.arange(3.),
+                  index=pd.date_range('2020-01-01', freq='D', periods=3, tz='UTC')
+                  ),
+        time_zone='Europe/Stockholm', freq='D'
+    )
+    pd_series = pd.Series(np.arange(3.),
+                          index=pd.date_range('2020-01-01',
+                                              freq='D', periods=3, tz='UTC'))
+    result_ts = ts + pd_series
+    assert result_ts.first == (
+        pd.Timestamp('2020-01-01').tz_localize('UTC').tz_convert('Europe/Stockholm'), 0.
+    )
+    assert result_ts.values == [0., 2., 4.]
+    assert result_ts.last == (
+        pd.Timestamp('2020-01-03').tz_localize('UTC').tz_convert('Europe/Stockholm'), 4.
+    )
+    pd.testing.assert_series_equal(
+        result_ts._series,
+        pd.Series([0., 2., 4.],
+                  index=pd.date_range('2020-01-01', freq='D', periods=3, tz='UTC')
+                        .tz_convert('Europe/Stockholm'))
+    )
+    assert result_ts.time_zone == pytz.timezone('Europe/Stockholm')
+
+
 def test_timestamp_series_add_list_error():
     ts = TimestampSeries(
         pd.Series(np.arange(3),
@@ -530,15 +690,94 @@ def test_timestamp_series_add_scalar():
     assert result_ts.values == [1., 2., 3.]
 
 
-def test_timestamp_series_multiply_scalar():
+def test_timestamp_series_multiply_scalar(default_timestamp_series):
+    result_ts = default_timestamp_series * 2
+    assert result_ts.values == [0., 2., 4.]
+
+
+def test_timestamp_series_multiply_pint_scalar(default_timestamp_series):
     ts = TimestampSeries(
         pd.Series(np.arange(3),
-                  index=pd.date_range('2020-01-01', freq='D', periods=3, tz='UTC')
+                  index=pd.date_range('2020-01-01', freq='D', periods=3)
                   ),
-        time_zone='UTC', freq='D'
+        unit='m'
     )
-    result_ts = ts * 2
+    result_ts = ts * (2 * ureg.Unit('m'))
     assert result_ts.values == [0., 2., 4.]
+    assert result_ts.unit == ureg.Unit('m^2')
+
+
+def test_timestamp_series_div_scalar(default_timestamp_series):
+    result_ts = default_timestamp_series / 2
+    assert result_ts.values == [0., .5, 1.]
+
+
+@pytest.mark.skip('should work')  # Todo: to be reported @pint_pandas
+def test_timestamp_series_floordiv_scalar(default_timestamp_series):
+    result_ts = default_timestamp_series // 2
+    assert result_ts.values == [0., 0., 1.]
+
+
+def test_timestamp_series_div_pint_scalar():
+    ts = TimestampSeries(
+        pd.Series(np.arange(3),
+                  index=pd.date_range('2020-01-01', freq='D', periods=3)
+                  ),
+        unit='m^2'
+    )
+    result_ts = ts / (2 * ureg.Unit('m'))
+    assert result_ts.values == [0., .5, 1.]
+    assert result_ts.unit == ureg.Unit('m')
+
+
+def test_timestamp_series_subtract_list():
+    ts = TimestampSeries(
+        pd.Series(np.arange(3),
+                  index=pd.date_range('2020-01-01', freq='D', periods=3)
+                  ))
+    result_ts = ts - [0., 1., 2.]
+    assert result_ts.values == [0., 0., 0.]
+
+
+def test_timestamp_series_subtract_pd_series_mismatch():
+    ts = TimestampSeries(
+        pd.Series(np.arange(3),
+                  index=pd.date_range('2020-01-01', freq='D', periods=3)),
+    )
+    result_ts = ts - pd.Series(np.arange(2),
+                               index=pd.date_range('2020-01-01', freq='D', periods=2))
+    assert result_ts.timestamps == ts.timestamps
+    assert result_ts.values == [0., 0., 2.]
+
+
+def test_get_item_positional_index(default_timestamp_series):
+    assert default_timestamp_series[0] == 0 * ureg.Unit('m')
+
+
+def test_get_item_datetime_index(default_timestamp_series):
+    assert default_timestamp_series[default_timestamp_series.start] == 0 * ureg.Unit('m')
+
+
+def test_get_item_naive_datetime_index(default_timestamp_series):
+    assert default_timestamp_series[
+               default_timestamp_series.start.replace(tzinfo=None)] == 0 * ureg.Unit('m')
+
+
+def test_loop(default_timestamp_series):
+    for idx, item in enumerate(default_timestamp_series):
+        assert item == idx * ureg.Unit('m')
+
+
+def test_eq(default_timestamp_series):
+    ts1 = TimestampSeries.create_from_tuples(
+        [(pd.Timestamp(2020, 1, 1).tz_localize('UTC').tz_convert('EST'), 1.)],
+        unit='km', freq=None,
+    )
+    ts2 = TimestampSeries.create_from_tuples(
+        [(pd.Timestamp(2020, 1, 1).tz_localize('UTC').tz_convert('GMT'), 1000.)],
+        unit='m', freq=None,
+    )
+    assert ts1 == ts2
 
 
 def test_timestamp_series_convert_unit_from_none():
@@ -579,14 +818,89 @@ def test_timestamp_series_fill_gaps():
         dt.datetime(2020, 3, 1, 17, 0, 0),
     ]
     values = [1, 1, 1]
-    ts = TimestampSeries.create_from_lists(timestamps, values, freq=pd.offsets.Hour())
-    ts = ts.fill_gaps(timestamps[0] - dt.timedelta(hours=1),
-                      timestamps[-1] + dt.timedelta(hours=1), value=2)
+    ts = TimestampSeries.create_from_lists(timestamps, values, freq=pd.offsets.Hour(),
+                                           time_zone='Europe/Berlin')
+    ts = ts.fill_gaps(ts.start - dt.timedelta(hours=1),
+                      ts.end + dt.timedelta(hours=1), value=2)
+    assert len(ts) == 5
+    assert ts.start == ts.time_zone.localize(timestamps[0]) - dt.timedelta(hours=1)
+    assert ts.end == ts.time_zone.localize(timestamps[-1]) + dt.timedelta(hours=1)
+    assert ts[timestamps[0] - dt.timedelta(hours=1)] == 2
+    assert ts[timestamps[-1] + dt.timedelta(hours=1)] == 2
+
+
+def test_timestamp_series_fill_gaps_start_and_end_different_timezone():
+    timestamps = [
+        dt.datetime(2020, 3, 1, 15, 0, 0),
+        dt.datetime(2020, 3, 1, 16, 0, 0),
+        dt.datetime(2020, 3, 1, 17, 0, 0),
+    ]
+    values = [1, 1, 1]
+    ts = TimestampSeries.create_from_lists(
+        timestamps,
+        values,
+        freq=pd.offsets.Hour(),
+        time_zone="UTC"
+    )
+    with pytest.raises(ValueError):
+        ts.fill_gaps(
+            pytz.timezone('Europe/Berlin').localize(timestamps[0]) - dt.timedelta(hours=1),
+            pytz.timezone('Europe/Berlin').localize(timestamps[-1]) + dt.timedelta(hours=1),
+            value=2
+        )
+
+
+def test_timestamp_series_fill_gaps_start_and_end_no_timezone():
+    timestamps = [
+        dt.datetime(2020, 3, 1, 15, 0, 0),
+        dt.datetime(2020, 3, 1, 16, 0, 0),
+        dt.datetime(2020, 3, 1, 17, 0, 0),
+    ]
+    values = [1, 1, 1]
+    ts = TimestampSeries.create_from_lists(
+        timestamps,
+        values,
+        freq=pd.offsets.Hour(),
+        time_zone="Europe/Berlin"
+    )
+    ts.fill_gaps(
+        timestamps[0] - dt.timedelta(hours=1),
+        timestamps[-1] + dt.timedelta(hours=1),
+        value=2
+    )
     assert len(ts) == 5
     assert ts[timestamps[0] - dt.timedelta(hours=1)] == 2
     assert ts[timestamps[-1] + dt.timedelta(hours=1)] == 2
 
 
+def test_timestamp_series_fill_gaps_no_start_and_end():
+    timestamps = [
+        dt.datetime(2020, 3, 1, 15, 0, 0),
+        dt.datetime(2020, 3, 1, 16, 0, 0),
+        dt.datetime(2020, 3, 1, 17, 0, 0),
+    ]
+    values = [1, 1, np.nan]
+    ts = TimestampSeries.create_from_lists(timestamps, values, freq=pd.offsets.Hour())
+    ts = ts.fill_gaps(value=2)
+    assert len(ts) == 3
+    assert ts.timestamps == timestamps
+    assert ts.values == [1, 1, 2]
+
+
+def test_timestamp_series_fill_gaps_with_unit(default_timestamp_series):
+    default_timestamp_series.fill_gaps(
+        end=default_timestamp_series.timestamps[-1] + default_timestamp_series.freq,
+        value=0.0
+    )
+    assert default_timestamp_series.values == [0.0, 1.0, 2.0, 0.0]
+    assert default_timestamp_series._series.dtype == default_timestamp_series.unit
+
+
+def test_timestamp_series_fill_gaps_empty_series(empty_timestamp_series):
+    with pytest.raises(ValueError):
+        empty_timestamp_series.fill_gaps()
+
+
 def test_timestamp_series_get_gaps():
     timestamps = [
         dt.datetime(2020, 3, 1, 15, 0, 0),
@@ -603,13 +917,99 @@ def test_timestamp_series_get_gaps():
     ]
 
 
-def test_timestamp_series_resample():
+def test_timestamp_series_get_gaps_start_and_end_different_timezone():
+    timestamps = [
+        dt.datetime(2020, 3, 1, 15, 0, 0),
+        dt.datetime(2020, 3, 1, 16, 0, 0),
+        dt.datetime(2020, 3, 1, 17, 0, 0),
+    ]
+    values = [1, 1, 1]
+    ts = TimestampSeries.create_from_lists(
+        timestamps,
+        values,
+        freq=pd.offsets.Hour(),
+        time_zone="UTC"
+    )
+    with pytest.raises(ValueError):
+        ts.get_gaps(
+            pytz.timezone('Europe/Berlin').localize(timestamps[0]) - dt.timedelta(hours=1),
+            pytz.timezone('Europe/Berlin').localize(timestamps[-1]) + dt.timedelta(hours=1),
+        )
+
+
+def test_timestamp_series_get_gaps_start_and_end_no_timezone():
+    timestamps = [
+        dt.datetime(2020, 3, 1, 15, 0, 0),
+        dt.datetime(2020, 3, 1, 16, 0, 0),
+        dt.datetime(2020, 3, 1, 17, 0, 0),
+    ]
+    values = [1, 1, 1]
+    ts = TimestampSeries.create_from_lists(
+        timestamps,
+        values,
+        freq=pd.offsets.Hour(),
+        time_zone="Europe/Berlin"
+    )
+    gaps = ts.get_gaps(
+        timestamps[0] - dt.timedelta(hours=1),
+        timestamps[-1] + dt.timedelta(hours=1),
+    )
+    assert gaps == [
+        pytz.timezone('Europe/Berlin').localize(timestamps[0]) - dt.timedelta(
+            hours=1),
+        pytz.timezone('Europe/Berlin').localize(timestamps[-1]) + dt.timedelta(
+            hours=1),
+    ]
+
+
+def test_timestamp_series_get_gaps_no_start_and_end():
+    timestamps = [
+        dt.datetime(2020, 3, 1, 15, 0, 0),
+        dt.datetime(2020, 3, 1, 16, 0, 0),
+        dt.datetime(2020, 3, 1, 17, 0, 0),
+    ]
+    values = [np.nan, 1, 1]
+    ts = TimestampSeries.create_from_lists(timestamps, values, freq=pd.offsets.Hour())
+    gaps = ts.get_gaps()
+    assert gaps == [
+        timestamps[0]
+    ]
+
+
+def test_timestamp_series_get_gaps_empty_series(empty_timestamp_series):
+    with pytest.raises(ValueError):
+        empty_timestamp_series.get_gaps()
+
+
+def test_timestamp_series_get_gaps_with_unit(default_timestamp_series):
+    end = default_timestamp_series.timestamps[-1] + default_timestamp_series.freq
+    gaps = default_timestamp_series.get_gaps(
+        end=end,
+    )
+    assert gaps == [end]
+
+
+def test_timestamp_series_resample_str_method():
     ts = TimestampSeries(
         pd.Series(np.ones(48),
                   index=pd.date_range('2020-01-01', freq='H', periods=48, tz='CET')
                   ),
-        time_zone='CET', freq='H'
+        time_zone='CET', freq='H', unit='m',
     )
     ts = ts.resample('D', 'sum')
     assert ts.freq == pd.offsets.Day()
     assert ts.values == [24, 24]
+    assert ts.unit == ureg.Unit('m')
+
+
+def test_timestamp_series_resample_with_function_and_nan():
+    ts = TimestampSeries(
+        pd.Series(np.repeat(np.nan, 48),
+                  index=pd.date_range('2020-01-01', freq='H', periods=48, tz='CET')
+                  ),
+        time_zone='CET', freq='H', unit='m',
+    )
+    ts = ts.resample('D', np.mean)
+    assert ts.freq == pd.offsets.Day()
+    assert np.all(np.isnan(ts.values))
+    assert ts.unit == ureg.Unit('m')
