@@ -23,7 +23,7 @@ class TimestampSeries(UnitMixin, TimeZoneMixin, FrequencyMixin, BaseTimeSeries):
     @staticmethod
     def create_null_timeseries(start, end, freq, unit=None, time_zone='infer'):
         """
-        create a `TimestampSeries`-object from `start` to `end``with NaN-values
+        create a `TimestampSeries`-object from `start` to `end` with NaN-values
 
         :param str/datetime.datetime/pandas.Timestamp start: the start timestamp of the
             series (included)
@@ -39,7 +39,7 @@ class TimestampSeries(UnitMixin, TimeZoneMixin, FrequencyMixin, BaseTimeSeries):
             or a tzinfo-object, pass `'infer'` if you want the time zone to be derived
             from `start` and `end`
         :return: a new TimestampSeries-object
-        :rtype:TimestampSeries
+        :rtype: TimestampSeries
         """
         return TimestampSeries.create_constant_timeseries(
             start, end, np.NaN, freq, unit, time_zone=time_zone)
@@ -90,7 +90,7 @@ class TimestampSeries(UnitMixin, TimeZoneMixin, FrequencyMixin, BaseTimeSeries):
             and many more
         :param str/tzinfo time_zone: the name of the time zone, (see `IANA <https://www.iana.org/time-zones>`_)
             or a tzinfo-object, pass `'infer'` if you want the time zone to be derived
-            from `start` and `end`
+            by the timestamps
         :return: a new TimestampSeries-object
         :rtype: TimestampSeries
         """
@@ -115,7 +115,7 @@ class TimestampSeries(UnitMixin, TimeZoneMixin, FrequencyMixin, BaseTimeSeries):
             and many more
         :param str/tzinfo time_zone: the name of the time zone, (see `IANA <https://www.iana.org/time-zones>`_)
             or a tzinfo-object, pass `'infer'` if you want the time zone to be derived
-            from `start` and `end`
+            by the timestamps
         :return: a new TimestampSeries-object
         :rtype: TimestampSeries
         """
@@ -129,7 +129,8 @@ class TimestampSeries(UnitMixin, TimeZoneMixin, FrequencyMixin, BaseTimeSeries):
         create a `TimestampSeries`-object from a dict timestamps as keys and values as
         values
 
-        :param list tuples: list of tuples holding timestamp and value
+        :param dict dictionary: dict with timestamps as keys and timeseries-values as
+            dict-values
         :param str/datetime.timedelta/pandas.Offset/pandas.Timedelta freq:
             the frequency of the timestamp series, `pandas offset aliases <https://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html#offset-aliases>`_
             supported, pass `'infer'` if you want the frequency to be derived by the
@@ -139,7 +140,7 @@ class TimestampSeries(UnitMixin, TimeZoneMixin, FrequencyMixin, BaseTimeSeries):
             and many more
         :param str/tzinfo time_zone: the name of the time zone, (see `IANA <https://www.iana.org/time-zones>`_)
             or a tzinfo-object, pass `'infer'` if you want the time zone to be derived
-            from `start` and `end`
+            by the timestamps
         :return: a new TimestampSeries-object
         :rtype: TimestampSeries
         """
@@ -147,8 +148,8 @@ class TimestampSeries(UnitMixin, TimeZoneMixin, FrequencyMixin, BaseTimeSeries):
         return TimestampSeries.create_from_pd_series(series, freq=freq, unit=unit,
                                                      time_zone=time_zone)
 
-    @classmethod
-    def create_from_pd_series(cls, series, freq='infer', unit=None, time_zone='infer'):
+    @staticmethod
+    def create_from_pd_series(series, freq='infer', unit=None, time_zone='infer'):
         """
         create a `TimestampSeries`-object from a pandas `Series` with `DatetimeIndex`
 
@@ -166,7 +167,7 @@ class TimestampSeries(UnitMixin, TimeZoneMixin, FrequencyMixin, BaseTimeSeries):
         :return: a new TimestampSeries-object
         :rtype: TimestampSeries
         """
-        return cls(series, freq=freq, unit=unit, time_zone=time_zone)
+        return TimestampSeries(series, freq=freq, unit=unit, time_zone=time_zone)
 
     # ------------------------------ constructor ----------------------------- #
 
@@ -233,6 +234,59 @@ class TimestampSeries(UnitMixin, TimeZoneMixin, FrequencyMixin, BaseTimeSeries):
 
     # ---------------------------- functionality ----------------------------- #
 
+    def map(self, func, dimensionless=True):
+        """
+        apply a custom function to each value of the series
+
+        :param function func: a function mapping a scalar to another scalar
+        :param bool dimensionless: if set to True, the mapping function takes
+            an argument of type Number (no unit, dimensionless). The resulting
+            timestamp series will keep the original unit. If set to False,
+            the mapping function takes an argument of type pint.Quantity.
+            The resulting timestamp series will have the unit of the mapped
+            values. Mapping values of one series to different units results in
+            an error. Mapping with dimensionless=False will result in a loop
+            and therefore perform slower.
+        :return: the series with mapped values
+        :rtype: TimestampSeries
+        """
+        if self.empty:
+            return self
+
+        if isinstance(self._series.dtype, PintType):
+            if dimensionless:
+                mapped_values = self._get_magnitude_series().apply(func).values
+                self._series = pd.Series(PintArray(mapped_values, dtype=self.unit),
+                                         index=self._series.index)
+            else:
+                mapped_values = list(map(func, self._series.values))
+                mapped_unit = mapped_values[0].u
+                if any(map(lambda x: x.u != mapped_unit, mapped_values)):
+                    raise ValueError("the mapped values do not have the same unit")
+                magnitudes = [v.magnitude for v in mapped_values]
+                self._series = pd.Series(PintArray(magnitudes, dtype=mapped_unit),
+                                         index=self._series.index)
+        else:
+            self._series = self._series.apply(func)
+        return self
+
+    def round(self, decimals):
+        """
+        round the values of the series
+
+        :param decimals: no of decimal places to round to
+        :return: the series with rounded values
+        :rtype: TimestampSeries
+        """
+        # ToDo: feature request at pint-pandas
+        if isinstance(self._series.dtype, PintType):
+            rounded_values = self._get_magnitude_series().values.round(decimals)
+            self._series = pd.Series(PintArray(rounded_values, dtype=self.unit),
+                                     index=self._series.index)
+        else:
+            self._series = self._series.round(decimals)
+        return self
+
     def append(self, value):
         """
         append a new value to a series with frequency
@@ -315,8 +369,25 @@ class TimestampSeries(UnitMixin, TimeZoneMixin, FrequencyMixin, BaseTimeSeries):
     def __eq__(self, other):
         if not isinstance(other, TimestampSeries):
             return False
-        return self.timestamps == other.timestamps \
-               and self.values == other.values
+        self_values = self.values
+        other_values = other.values
+        self_timestamps = self.timestamps
+        other_timestamps = other.timestamps
+        if self.unit:
+            self_values = self._series.pint.to_base_units().values.data
+        if other.unit:
+            other_values = other._series.pint.to_base_units().values.data
+        if self.time_zone:
+            tmp_self = copy.deepcopy(self)
+            tmp_self.convert_time_zone('UTC')
+            self_timestamps = tmp_self.timestamps
+        if other.time_zone:
+            tmp_other = copy.deepcopy(other)
+            tmp_other.convert_time_zone('UTC')
+            other_timestamps = tmp_other.timestamps
+
+        return self_timestamps == other_timestamps \
+               and self_values == other_values
 
     def __setitem__(self, key, value):
         raise NotImplementedError()
@@ -328,7 +399,7 @@ class TimestampSeries(UnitMixin, TimeZoneMixin, FrequencyMixin, BaseTimeSeries):
             return self._basic_calc_time_series(operation, other, **kwargs)
         elif isinstance(other, pd.Series):
             return self._basic_calc_pd_series(operation, other, **kwargs)
-        elif isinstance(other, (collections.Sequence, np.ndarray, PintArray)):
+        elif isinstance(other, (collections.abc.Sequence, np.ndarray, PintArray)):
             return self._basic_calc_collection(operation, other)
         else:
             return self._basic_calc_scalar(operation, other)
@@ -343,6 +414,7 @@ class TimestampSeries(UnitMixin, TimeZoneMixin, FrequencyMixin, BaseTimeSeries):
             warnings.warn("timestamps do not match, values are auto-filled")
         tmp_series._series = getattr(tmp_series._series, operation)(
             other._series, **kwargs)
+        tmp_series.convert_time_zone(self.time_zone)
         return tmp_series
 
     def _basic_calc_pd_series(self, operation, other, **kwargs):
@@ -354,8 +426,11 @@ class TimestampSeries(UnitMixin, TimeZoneMixin, FrequencyMixin, BaseTimeSeries):
         if not self._series.index.equals(other.index):
             warnings.warn("timestamps do not match, values are auto-filled")
         tmp_series._series = getattr(tmp_series._series, operation)(other, **kwargs)
+        # enforce resulting TimestampSeries' time zone to be equal to initial
+        # TimestampSeries (self)
+        tmp_series.convert_time_zone(self.time_zone)
         if isinstance(tmp_series._series.dtype, PintType):
-            tmp_series._unit = tmp_series._series.dtype.units
+            tmp_series._unit = tmp_series._series.pint.u
         return tmp_series
 
     def _basic_calc_collection(self, operation, other):
@@ -366,14 +441,16 @@ class TimestampSeries(UnitMixin, TimeZoneMixin, FrequencyMixin, BaseTimeSeries):
             raise ValueError("sequence contains non-numeric values")
         tmp_series._series = getattr(tmp_series._series, operation)(other)
         if isinstance(tmp_series._series.dtype, PintType):
-            tmp_series._unit = tmp_series._series.dtype.units
+            tmp_series._unit = tmp_series._series.pint.u
         return tmp_series
 
     def _basic_calc_scalar(self, operation, other):
         tmp_series = copy.deepcopy(self)
-        if not isinstance(other, numbers.Number):
+        if not isinstance(other, (numbers.Number, Quantity)):
             raise ValueError('value is not numeric')
         tmp_series._series = getattr(tmp_series._series, operation)(other)
+        if isinstance(other, Quantity):
+            tmp_series._unit = tmp_series._series.pint.u
         return tmp_series
 
     # ---------------------------- validation ------------------------------- #
